@@ -33,6 +33,7 @@
 #include "about.h"
 #include "configuration.h"
 #include "settings_dialog.h"
+#include "statistics_dialog.h"
 #include "../config.h"
 #include <iostream>
 #include <sstream>
@@ -69,7 +70,6 @@ namespace dice {
 #define VER_DICE_SPACER 10
 
 using namespace main_frame;
-const wxEventType wxEVT_ENABLE_ROLL = wxNewEventType();
 
 MainFrame::MainFrame(const wxString& title, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE)
         : wxFrame(NULL, wxID_ANY, title, wxDefaultPosition, size, style)
@@ -80,18 +80,22 @@ MainFrame::MainFrame(const wxString& title, const wxSize& size = wxDefaultSize,
 	#ifndef PORTABLE
 		wxString config_file = wxStandardPaths::Get().GetUserConfigDir() + wxT("/.openyahtzee");
 		wxString old_config_file = wxFileName::GetHomeDir() + wxT("/.OpenYahtzee");
-		//rename old configuration file
-		if (wxFileExists(old_config_file)) {
+		// Move the configuration file from the old name to the new one
+		// if there isn't a file with that name already.
+		if (wxFileExists(old_config_file) && !wxFileExists(config_file)) {
 			wxRenameFile(old_config_file,config_file,false);
 		}
 	#else
-		wxString config_file = wxStandardPaths::Get().GetExecutablePath() + wxT("/openyahtzee.dat");
+		wxFileName tmp_config_file(wxStandardPaths::Get().GetExecutablePath());
+		tmp_config_file.SetFullName(wxT("openyahtzee.dat"));
+		wxString config_file = tmp_config_file.GetFullPath();
 	#endif
 
 	
 	std::string config_file_str;
 	config_file_str = config_file.utf8_str();
-	m_config = new configuration::Configuration(config_file_str);
+	m_config.reset(new configuration::Configuration(config_file_str));
+	m_stats.reset(new statistics::Statistics(m_config.get()));
 
 	m_evt_handler = new MainFrameEvtHandler(this);
 	
@@ -125,7 +129,6 @@ MainFrame::MainFrame(const wxString& title, const wxSize& size = wxDefaultSize,
 	m_yahtzee = false;
 	m_yahtzeebonus = false;
 	m_numofplaysleft = 13;
-	m_skiproll = false; // the roll button hasn't been pressed yet, no need for skipping
 }
 
 /**
@@ -148,6 +151,7 @@ void MainFrame::AddMenus()
 	gameMenu->Append(wxID_NEW,wxT("&New Game\tF2"));
 	gameMenu->Append(wxID_UNDO,wxT("&Undo\tCtrl+Z"));
 	gameMenu->Append(ID_SHOWHIGHSCORE,wxT("Show Highscores\tCtrl+H"));
+	//gameMenu->Append(ID_STATISTICS,wxT("Statistics..."));
 	gameMenu->Append(ID_SETTINGS,wxT("Settings"));
 	gameMenu->Append(wxID_EXIT);
 	
@@ -345,11 +349,11 @@ void MainFrame::ConnectEventTable()
 	Connect(wxID_NEW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnNewGame));
 	Connect(wxID_UNDO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnUndo));
 	Connect(ID_SHOWHIGHSCORE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnShowHighscore));
+	Connect(ID_STATISTICS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnStatistics));
 	Connect(ID_SETTINGS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSettings));
 	//END connecting the menu items' events
 
-	Connect(ID_ROLL, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler (MainFrame::OnRollButton));
-	Connect(ID_ROLL, wxEVT_ENABLE_ROLL, wxCommandEventHandler (MainFrame::OnRollButton));
+	Connect(ID_ROLL, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler (MainFrame::DoubleRollLock));
 
 	//BEGIN connecting the scoreboard buttons to the events
 	Connect(ID_ACES,ID_SIXES, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler (MainFrame::OnUpperButtons));
@@ -517,13 +521,20 @@ inline void MainFrame::EnableUndo(int id)
  */
 void MainFrame::OnShowHighscore(wxCommandEvent& event)
 {
-	highscores_dialog::HighscoresDialog *dialog = new highscores_dialog::HighscoresDialog(this,m_config);
+	highscores_dialog::HighscoresDialog *dialog = new highscores_dialog::HighscoresDialog(this, m_config.get());
 
 	dialog->ShowModal();
 
 	delete dialog;
 }
 
+void MainFrame::OnStatistics(wxCommandEvent &event)
+{
+	statistics_dialog::StatisticsDialog *dialog = new statistics_dialog::StatisticsDialog(this, m_stats.get());
+	dialog->ShowModal();
+	delete dialog;
+}
+
 /**
  * Shows the settings dialog. This event-handler is connected to Game->Settings
  * menu item.
@@ -531,7 +542,7 @@ void MainFrame::OnShowHighscore(wxCommandEvent& event)
  */
 void MainFrame::OnSettings( wxCommandEvent& event)
 {
-	settings_dialog::SettingsDialog* settings_dialog = new settings_dialog::SettingsDialog(this, m_config);
+	settings_dialog::SettingsDialog* settings_dialog = new settings_dialog::SettingsDialog(this, m_config.get());
 
 	if (settings_dialog->ShowModal()!=wxID_OK) {
 		delete settings_dialog;
@@ -553,19 +564,14 @@ void MainFrame::OnSettings( wxCommandEvent& event)
  * status of the "keep" checkboxes.
  * \param event 
  */
-void MainFrame::OnRollButton (wxCommandEvent& event)
+void MainFrame::OnRollButton ()
 {
 	short int dice[5];	//holds the dices score
 	
-	if (event.GetEventType() == wxEVT_IDLE) {
-		m_skiproll = false;
-		Disconnect(wxEVT_IDLE,  wxCommandEventHandler(MainFrame::OnRollButton));
-		return;
-	}
-	
-	//skip rolling the dice if the user accidently rolled the dice before they finished spinning.
-	if (m_skiproll) return;
-	m_skiproll = true;
+	// If this is the first roll of the game, notify the statistics
+	// object
+	if (m_numofplaysleft == 13 && m_rolls == 3)
+		m_stats->game_started();
 
 	//fill the dice array with the old values
 	dice[0]=m_score_dice.GetDice(1);
@@ -626,8 +632,29 @@ void MainFrame::OnRollButton (wxCommandEvent& event)
 	//we rolled the dices so undoing isn't allowed
 	(GetMenuBar()->FindItem(wxID_UNDO))->Enable(false);
 	m_yahtzeebonus = false; //if we scored yahtzee bonus before we don't care anymore.
+}
+
+void MainFrame::DoubleRollLock(wxCommandEvent& event)
+{
+	/* The function works by connecting to idle event, and raising a
+	 * flag when it process a valid click on the Roll button. When the
+	 * another click happens and the flag is up, it means it is
+	 * accidental. When the idle events is triggered it means that we
+	 * no longer roll the dice, so we lower the flag.
+	 */
+	static bool skip_roll = false;
+	if (event.GetEventType() == wxEVT_IDLE) {
+		skip_roll = false;
+		Disconnect(wxEVT_IDLE,  wxCommandEventHandler(MainFrame::DoubleRollLock));
+		return;
+	}
 	
-	Connect(wxEVT_IDLE, wxCommandEventHandler(MainFrame::OnRollButton));
+	if (skip_roll)
+		return;
+	skip_roll = true;
+	
+	Connect(wxEVT_IDLE, wxCommandEventHandler(MainFrame::DoubleRollLock));
+	OnRollButton();
 }
 
 /**
@@ -643,39 +670,39 @@ void MainFrame::OnUpperButtons (wxCommandEvent& event)
 {
 	wxString out;
 	short int temp;
-	if(m_rolls < 3){
-		YahtzeeBonus();
-		switch(event.GetId()) {
-		case ID_ACES:
-			temp = m_score_dice.Aces();
-			break;
-		case ID_TWOS:
-			temp = m_score_dice.Twos();
-			break;
-		case ID_THREES:
-			temp = m_score_dice.Threes();
-			break;
-		case ID_FOURS:
-			temp = m_score_dice.Fours();
-			break;
-		case ID_FIVES:
-			temp = m_score_dice.Fives();
-			break;
-		case ID_SIXES:
-			temp = m_score_dice.Sixes();
-			break;
-		}
-	
-		out.Printf(wxT("%i"),temp);
-		wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(event.GetId() - ID_ACES + ID_ACESTEXT));
-		text_ctrl->SetValue(out);
-		text_ctrl->SetBackgroundColour(*wxWHITE);
-		text_ctrl->Refresh();
-		
-		PostScore(event.GetId());
+	if (!IsValidClick())
+		return;
+
+	YahtzeeBonus();
+
+	switch(event.GetId()) {
+	case ID_ACES:
+		temp = m_score_dice.Aces();
+		break;
+	case ID_TWOS:
+		temp = m_score_dice.Twos();
+		break;
+	case ID_THREES:
+		temp = m_score_dice.Threes();
+		break;
+	case ID_FOURS:
+		temp = m_score_dice.Fours();
+		break;
+	case ID_FIVES:
+		temp = m_score_dice.Fives();
+		break;
+	case ID_SIXES:
+		temp = m_score_dice.Sixes();
+		break;
 	}
-	else 
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	
+	out.Printf(wxT("%i"),temp);
+	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(event.GetId() - ID_ACES + ID_ACESTEXT));
+	text_ctrl->SetValue(out);
+	text_ctrl->SetBackgroundColour(*wxWHITE);
+	text_ctrl->Refresh();
+	
+	PostScore(event.GetId());
 }
 
 /**
@@ -684,10 +711,9 @@ void MainFrame::OnUpperButtons (wxCommandEvent& event)
  */
 void MainFrame::On3ofakindButton(wxCommandEvent& event)
 {
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
+	
 	YahtzeeBonus();
 	wxString out;
 	
@@ -706,10 +732,9 @@ void MainFrame::On3ofakindButton(wxCommandEvent& event)
  */
 void MainFrame::On4ofakindButton(wxCommandEvent& event)
 {
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
+	
 	YahtzeeBonus();
 	wxString out;
 
@@ -730,10 +755,9 @@ void MainFrame::OnFullHouseButton(wxCommandEvent& event)
 {
 	wxString out;
 
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
+	
 	YahtzeeBonus();
 
 	out.Printf(wxT("%i"), m_score_dice.FullHouse());
@@ -753,10 +777,8 @@ void MainFrame::OnSmallSequenceButton(wxCommandEvent& event)
 {
 	wxString out;
 
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
 
 	YahtzeeBonus();
 	
@@ -777,10 +799,8 @@ void MainFrame::OnLargeSequenceButton(wxCommandEvent& event)
 {
 	wxString out;
 	
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
 
 	YahtzeeBonus();
 
@@ -802,10 +822,8 @@ void MainFrame::OnYahtzeeButton(wxCommandEvent& event)
 {
 	wxString out;
 	
-	if(m_rolls>=3) {
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
 		return;
-	}
 	
 	if (m_score_dice.IsYahtzee()) m_yahtzee = true;
 	out.Printf(wxT("%i"), m_score_dice.Yahtzee());
@@ -824,19 +842,17 @@ void MainFrame::OnYahtzeeButton(wxCommandEvent& event)
 void MainFrame::OnChanceButton (wxCommandEvent& event)
 {
 	wxString out;
-	if(m_rolls < 3){
-		YahtzeeBonus();
-	
-		out.Printf(wxT("%i"),m_score_dice.Chance());
-		wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_CHANCETEXT));
-		text_ctrl->SetValue(out);
-		text_ctrl->SetBackgroundColour(*wxWHITE);
-		text_ctrl->Refresh();
-		PostScore(event.GetId());
-	}
-	else 
-		wxMessageBox(wxT("First you need to roll, and after you roll you may score"), wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+	if (!IsValidClick())
+		return;
 
+	YahtzeeBonus();
+
+	out.Printf(wxT("%i"),m_score_dice.Chance());
+	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_CHANCETEXT));
+	text_ctrl->SetValue(out);
+	text_ctrl->SetBackgroundColour(*wxWHITE);
+	text_ctrl->Refresh();
+	PostScore(event.GetId());
 }
 
 /**
@@ -977,6 +993,8 @@ void MainFrame::EndofGame()
 	tempstr.Printf(wxT("Your final score is %i points!"),lowerscore+upperscore);
 	wxMessageBox(tempstr, wxT("Game Ended"), wxOK | wxICON_INFORMATION, this);
 
+	m_stats->game_finished(lowerscore+upperscore);
+
 	//submit to high score
 	HighScoreHandler(lowerscore+upperscore);
 	
@@ -1016,7 +1034,7 @@ void MainFrame::HighScoreHandler(int score)
 	int rank = m_config->submitHighscore(score, name, date);
 
 	//now show the high score table
-	highscores_dialog::HighscoresDialog *dialog = new highscores_dialog::HighscoresDialog(this,m_config,rank);
+	highscores_dialog::HighscoresDialog *dialog = new highscores_dialog::HighscoresDialog(this, m_config.get(),rank);
 
 	dialog->ShowModal();
 
@@ -1327,9 +1345,18 @@ void MainFrameEvtHandler::OnScoreMouseLeave (wxMouseEvent& event)
 	event.Skip(); //allow default proccesing
 }
 
+bool MainFrame::IsValidClick()
+{
+	if (m_rolls >= 3) {
+		wxMessageBox(wxT("First you need to roll, and after you roll you may score."),
+			wxT("Open Yahtzee"), wxOK | wxICON_INFORMATION, this);
+		return false;
+	}
+	return true;
+}
+
 MainFrame::~MainFrame() {
 	// free pointers
-	delete m_config;
 	delete m_evt_handler;
 
 	for (int i = 0; i<6; i++) {
